#!/bin/bash
{% if extra_job_settings is defined %}
{% for setting in extra_job_settings %}
{{ setting }}
{% endfor %}
{% endif %}
{{ job_runner }} apptainer exec {{ container_wrap_image }} {{ job_file }} {{ job_args }}